% Taken from Dr. Keogh's image2bmp.m, modified by Adam Meadows

% function: images2ts
% description: convert a cell array of images into a collection of time
% series
% author: Li Wei & Xiaopeng Xi (modified by Adam Meadows)
% last updated: 06/05/2006
%
% - add an input parameter 'do_plot'
%                           do_plot = 1, draw the extraction process
%                           otherwise,   don't draw
% - add two input parameters 'method'
%                           method =0, centroid extraction
%                           method =1, angle extraction
%                           'option'
%                           option = 1, nomalize and interpolate to the same length
%                           otherwise, don't do it
% - update the profile_extraction function (using the distance to centroid
% now) and fix a backtrace bug caused by noise

function freq = images2ts(imgs, do_plot, option, method)

close all;

index = 1;
seq_grup = zeros(1,1);          % return value of empty folder case
seqcoor_grup = zeros(1,1);      % store the value of profile coordinates

for i = 1 : length(imgs)
    
    old = imgs{i};

    % ##### rescale the images ####
    sizevector = size(old);
    num_of_dim = ndims(old);
    newim = old;
    if (num_of_dim == 3)
        if (sizevector(1) ~= sizevector(2))
            if (sizevector(1) > sizevector(2))                        
                newim(:, sizevector(2)+1:sizevector(1), :) = 0;                        
            else                        
                newim(sizevector(1)+1:sizevector(2), :, :) = 0;
            end
        end
    end
    if (num_of_dim == 2)
        if (sizevector(1) ~= sizevector(2))
            if (sizevector(1) > sizevector(2))                        
                new(:, sizevector(2)+1:sizevector(1)) = 0;                        
            else                        
                newim(sizevector(1)+1:sizevector(2), :) = 0;
            end
        end
    end   
    index = index + 1;

    [seq, seqcoor] = profile_extraction(old, do_plot, method);

    %keep track of the size of Timeseries
    sizes = size(seq);
    seq(sizes(2)+1) = sizes(2); 

    %keep track of the size of coordinate
    sizes2 = size(seqcoor);
    seqcoor(sizes2(2)+1) = sizes2(2); 


    %====== Adds the newly generated time series -- seq into the 
    %existing time series -- seq_grup ===================
    [newseq_num, max_size] = size(seq);
    if newseq_num == 0
        continue;
    end
    oldsize = size(seq_grup);
    if max_size > oldsize(2)    % all the previous stored seq need to be restored
        if oldsize(2) == 1      % seq_grup contains no useful data yet
            seq_grup = seq;     % reallocate the seq_grup
        else
            tmp = zeros(newseq_num+oldsize(1), max_size);   % already has values inside seq_grup
            padnum = max_size - oldsize(2);                 % get the pad array for the new storation
            pad = zeros(1,padnum);
            for i = 1:oldsize(1)
                tmp(i,:) = [seq_grup(i,:),pad];
                tmp(i,oldsize(2)) = [0];                    % clear old size info
                tmp(i,max_size) = seq_grup(i,oldsize(2));   % move the size info backwards
            end
            tmp(oldsize(1)+1:oldsize(1)+newseq_num,:) = seq;
            seq_grup = tmp;
        end                 
    else                    %no need to reallocate the original seq_grup, just copy the new seq into the new rows of it.
        padnum = oldsize(2) - max_size; % the new seq need be expanded by pad
        pad = zeros(1,padnum);
        for padind = 1:padnum
            pad(padind) = 0;
        end;
        tmp = zeros(newseq_num, oldsize(2));
        for i = 1:newseq_num
            tmp(i,:) = [seq(i,:),pad];
            tmp(i,max_size) = [0];%clear old size info
            tmp(i,oldsize(2)) = seq(i,max_size);%move the size info backwards
        end;
        seq_grup(oldsize(1)+1:oldsize(1) + newseq_num,:) = tmp;
    end;

    % ====== Adds the newly generated coordinates -- seqcoor into the 
    % existing time series -- seqcoor_grup ===================
    [newseq_num, max_size] = size(seqcoor);
    if newseq_num == 0
        continue;
    end
    oldsize = size(seqcoor_grup);
    if max_size > oldsize(2)  % all the previous stored seq need to be restored
        if oldsize(2) == 1% seq_grup contains no useful data yet
            seqcoor_grup = seqcoor;%reallocate the seq_grup
        else
            tmp = zeros(newseq_num+oldsize(1), max_size);% already has values inside seq_grup
            padnum = max_size - oldsize(2);    % get the pad array for the new storation
            pad = zeros(1,padnum);
            for i = 1:oldsize(1)
                tmp(i,:) = [seqcoor_grup(i,:),pad];
                tmp(i,oldsize(2)) = [0];%clear old size info
                tmp(i,max_size) = seqcoor_grup(i,oldsize(2));%move the size info backwards
            end
            tmp(oldsize(1)+1:oldsize(1)+newseq_num,:) = seqcoor;
            seqcoor_grup = tmp;
        end;%ending up the case which need to expand the original seq_grup
    else %no need to reallocate the original seq_grup, just copy the new seq into the new rows of it.
        padnum = oldsize(2) - max_size; % the new seq need be expanded by pad
        pad = zeros(1,padnum);
        for padind = 1:padnum
            pad(padind) = 0;
        end
        tmp = zeros(newseq_num, oldsize(2));
        for i = 1:newseq_num
            tmp(i,:) = [seqcoor(i,:),pad];
            tmp(i,max_size) = [0];%clear old size info
            tmp(i,oldsize(2)) = seqcoor(i,max_size);%move the size info backwards
        end
        seqcoor_grup(oldsize(1)+1:oldsize(1) + newseq_num,:) = tmp;
    end              
end

% normalize seq_grup and resample them to be same length
if option == 1
    newseq_grup = resampleAndNorm(seq_grup);
    %save(freqfile, 'newseq_grup', '-ascii');
    freq = newseq_grup;
else
    %save(freqfile, 'seq_grup', '-ascii');    
    freq = seq_grup;
end


% =========== local function ===============
function [TimeSeriesSeq, seqcoor] = profile_extraction(image, do_plot, method)

I = image;

row = size(I,1);
col = size(I,2);

%TBD: for now assume color images
C = 1;
if C == 1
     J = rgb2gray(I);
     se0 = strel('line', 2, 90);
     se1 = strel('line', 2, 0);
     J = imdilate(J, [se0,se1]);
     J = imfill(J,'hole');
     seD = strel('diamond',1);
     J = imerode(J,seD);
     J = im2bw(J,0.2);
     J = edge(J, 'sobel',(graythresh(I) * .1));
    
     k1 = find(J==0);
     k2 = find(J==1);
     if size(k1,1) == 0 | size(k2,1) == 0
        J = rgb2gray(I);
        J = im2bw(J,0.6);
        se0 = strel('line', 1, 90);
        se1 = strel('line', 1, 0);
        J = imdilate(J, [se0,se1]);
        seD = strel('diamond',1);
        J = imerode(J,seD);
        J = medfilt2(J,[6 6]);
        J = edge(J, 'sobel',(graythresh(I) * .1));
    end;
else
    temp = double(I);
    J = edge(temp, 'sobel',(graythresh(temp) * .1));
end;

if do_plot == 1
    figure, subplot(2,1,1);
    imshow(J);
    hold on;
end;

%    Get the coordinate of edge:			
%   Scan according to the order up-down, left-right;
%   Treat the matrix as a sparse matrix; 'find' returns the x/y of non-0 pixels]
%   c = [i, j, v] to display i, j, v in three columns.

Leaf_edge = double(J);
[imgmatrix_x,imgmatrix_y] = size(Leaf_edge);

%allocate the all 0 matrix as same size as the 0/1 image, to later on check if the neighbour are selected out or not
Checking_matrix = zeros(imgmatrix_x,imgmatrix_y);
[i_coordinate,j_coordinate,v] = find(Leaf_edge); 
%plot(j_coordinate(20), i_coordinate(20), 'g+');

%    Start from the first non-0 value clock-wise to find the sequence of angles
%   (adjacent two pixels).eg : Check (i, j)'s neighbour. Order should  be: 
%   (i-1,j), (i-1,j+1), (i,j+1), (i+1,j+1), (i+1,j), (i+1, j-1), (i,j-1), (i-1,j-1). 
%   The neighbor that has checked before should be omitted from the 8 possible neighbors checking. 
%   Only 1/7 is selected out.
c = [i_coordinate, j_coordinate].'; %two rows, each column are the coordinate of the leaf pixel
num = nnz(Leaf_edge);
N =   [1 1 0 -1 -1 -1 0 1; 0 1 1 1 0 -1 -1 -1];
N1 =  [1 1 0 -1 -1 -1 0 1; 0 1 1 1 0 -1 -1 -1];
N11 = [-1 -1 0 1 1 1 0 -1; 0 -1 -1 -1 0 1 1 1];
N2 = [-2 -2 -2 -1  0  1  2  2  2  2  2  1  0  -1  -2 -2; 
       0 1  2  2  2  2  2  1  0 -1 -2 -2 -2  -2  -2 -1];
N3 = [-3 -3 -3 -3 -2 -1  0  1  2  3  3  3  3  3  3  3  2  1  0 -1 -2 -3 -3 -3;  
       0  1  2  3  3  3  3  3  3  3  2  1  0 -1 -2 -3 -3 -3 -3 -3 -3 -3 -2 -1];

%%%%%%   seq = timeseries(i_coordinate, j_coordinate);
% convert the coordinate of the edge to the sequence of edge shape.
%%%%%%   function seq = timeseries(x, y)
n = size(i_coordinate);
seq_coordinate = zeros(2,n(1));  % Preallocate matrix for clockwise ordered leaf pixels


%find non-zero pixel in the 8 neighbours
i = i_coordinate(1);
j = j_coordinate(1);
%plot(j,i,'b+');

findpixel = false;
while Leaf_edge(i,j) == 0
    total_nb_cand = size(N1,2);
    for nb_cand = 1 : total_nb_cand
        i1 = i + N1(1, nb_cand);
        j1 = j + N1(2, nb_cand);
        if (Leaf_edge(i1,j1)~=0)
            i = i1;
            j = j1;
            findpixel = true;
            break;
        end;
    end;
    if(findpixel == false)   % find non-zero pixels in the farther neighbours
        total_nb_cand = size(N2,2);
        for nb_cand = 1 : total_nb_cand
            i1 = i + N2(1,nb_cand);
            j1 = j + N2(2, nb_cand);
           if (Leaf_edge(i1,j1)~=0)
                i = i1;
                j = j1;
                findpixel = true;
                break;
            end;
        end;
    end;
    if(findpixel == false)
        total_nb_cand = size(N3,2);
        for nb_cand = 1 : total_nb_cand
            i1 = i + N3(1,nb_cand);
            j1 = j + N3(2, nb_cand);
           if (Leaf_edge(i1,j1)~=0)
                i = i1;
                j = j1;
                findpixel = true;
                break;
            end;
        end;    
    end;
    if(findpixel == false)
        disp('could not find a valid pixel in the near area, please check it!');
        return;
    end;
end;
if do_plot == 1
    subplot(2,1,1);
    plot(j,i, 'b+');
end;

prior_i = -1;
prior_j = -2;
start_i = i;
start_j = j;
a_order = 1;
back_trace = 1;

while a_order <= n(1)
  
    %%% look careful for this backtrace, need to modify wisely
    if (prior_i == i) & (prior_j == j) % need to expand the neighboring scanning.
        step_len = step_len + 1; %  neighbour distance
        if step_len > 3
            if (a_order - back_trace) < 1
                break;
            end;
            i = imgmatrix_x + 1 - seq_coordinate(2,a_order-back_trace);
            j = seq_coordinate(1, a_order-back_trace);
            back_trace = back_trace + 1;
            prior_i = i;
            prior_j = j;
            step_len =  1;
            %break;
        end;
    else 
        step_len = 1;
        back_trace = 1;
        if a_order > n(1)/2 && abs(start_i - i) < 3 && abs(j - start_j) < 3
            break;% get the round trip!
        end;
        seq_coordinate(1,a_order) = j;   % keep such info into matrix seq_coordinate
        seq_coordinate(2,a_order) = imgmatrix_x + 1 - i;
        if prior_i ~= -1 & prior_j ~= -2 % not the first time to enter this loop
            if j > prior_j | (i < prior_i & j == prior_j) % go right direction, need to change the direction of search
                N1 = N;
            elseif j < prior_j | (i > prior_i & j == prior_j)
                N1 = N11;
            end;
        end;
        prior_i = i;
        prior_j = j;
        if a_order~=1
            Checking_matrix(i,j) = 1;% specify this pixel has been visited
        end;
        a_order = a_order + 1;
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Draw the sketch based on the scanning
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% process, attention : matrix x y & img
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% coordinate x y
        if do_plot == 1      
            subplot(2,1,1);
             plot(j,i, 'r');
       end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
    end 
% scan clockwise to find the next leaf pixel
    %disp(step_len);
    
    switch (step_len)
    case 1, total_nb_cand = size(N1,2);
            for nb_cand = 1 : total_nb_cand
                nb_x = min(i + N1(1, nb_cand),imgmatrix_x);
                nb_y = min(j + N1(2, nb_cand),imgmatrix_y);
                if(nb_x <= 0)
                    nb_x = 1;
                end
                if(nb_y <= 0)
                    nb_y = 1;
                end
                if (Leaf_edge(nb_x, nb_y)~=0) & (Checking_matrix(nb_x, nb_y) == 0) % leaf and not been selected out before
                    i = nb_x;
                    j = nb_y;
                    break;% jump out the neighbour checking if this is the neighbouring leaf pixel
                end;    
            end; 
    case 2, total_nb_cand = size(N2,2);
            for nb_cand = 1 : total_nb_cand
                nb_x = min(i + N2(1, nb_cand),imgmatrix_x);
                nb_y = min(j + N2(2, nb_cand),imgmatrix_y);
                if(nb_x <= 0)
                    nb_x = 1;
                end
                if(nb_y <= 0)
                    nb_y = 1;
                end
                if (Leaf_edge(nb_x, nb_y)~=0) & (Checking_matrix(nb_x, nb_y) == 0)% leaf and not been selected out before
                    i = nb_x;
                    j = nb_y;
                    break;% jump out the neighbour checking if this is the neighbouring leaf pixel
                end;    
            end; 
    case 3, total_nb_cand = size(N3,2);
            for nb_cand = 1 : total_nb_cand
                nb_x = min(i + N3(1, nb_cand),imgmatrix_x);
                nb_y = min(j + N3(2, nb_cand),imgmatrix_y);
                if(nb_x <= 0)
                    nb_x = 1;
                end
                if(nb_y <= 0)
                    nb_y = 1;
                end
                if (Leaf_edge(nb_x, nb_y)~=0) & (Checking_matrix(nb_x, nb_y) == 0)% leaf and not been selected out before
                    i = nb_x;
                    j = nb_y;
                    break;% jump out the neighbour checking if this is the neighbouring leaf pixel
                end;    
            end; 
    end; % end switch
end% while

%output the difference between the original edge image with the one we scaned.

S = nonzeros(seq_coordinate);
seqcoor = S';
seq_num = size(seqcoor,2)/2;
TimeSeriesSeq = zeros(1,seq_num);

x_coor = seqcoor(1:2:end-1);
y_coor = seqcoor(2:2:end);
center_j = round(mean(x_coor));
center_i = imgmatrix_x + 1 - round(mean(y_coor));

if do_plot == 1
    subplot(2,1,1);
    plot(center_j, center_i, 'w+');
end;


if method == 1
  shift = 60;
  for Angle_index = 1 : seq_num
     LeftIndex = Angle_index - shift;
     if (LeftIndex < 1)
         LeftIndex = LeftIndex + seq_num;
     end;
    RightIndex = Angle_index + shift;
     if (RightIndex > seq_num)
         RightIndex = RightIndex - seq_num;
     end;
     Left = [x_coor(1,LeftIndex); y_coor(1,LeftIndex)];
     Middle = [x_coor(1,Angle_index); y_coor(1,Angle_index)];
     Right = [x_coor(1,RightIndex); y_coor(1,RightIndex)];
     TimeSeriesSeq(Angle_index) = angle_cal(Left, Middle, Right) - pi;% calculate the angle given 3 points
 end;
elseif method == 0
  for num_pt = 1 : seq_num
    temp_j = x_coor(1,num_pt);
    temp_i = imgmatrix_x + 1 - y_coor(1,num_pt);
    TimeSeriesSeq(num_pt) = sqrt((center_j - temp_j)*(center_j - temp_j) + (center_i - temp_i) * (center_i - temp_i));
  end;
end;

%%%%% plot the outline and its timeseries into one axes and adding lines to connect the angles up.
if do_plot == 1
    
    [x,y] = size(TimeSeriesSeq);
    TSProcess = TimeSeriesSeq;
    TSProcess(1,:) = (TSProcess(1,:) - mean(TSProcess(1,:))) / std(TSProcess(1,:));
    
    for xindex = 1:y
         xaxes1(xindex) = xindex;
     end;
    xaxes = (xaxes1 - mean(xaxes1))/std(xaxes1);

    
    %figure('Name','Object & TimeSeires'),
    subplot(2,1,2);
    
    
    plot(xaxes, TSProcess);
    hold on,
    imgx_shrink = 10;
    imgy_shrink = 2;
     
     x_coordinate = (seq_coordinate(1,1:seq_num) - mean(seq_coordinate(1,1:seq_num)))...
         /std(seq_coordinate(1,1:seq_num))/imgx_shrink ;
     y_coordinate = (seq_coordinate(2,1:seq_num) - mean(seq_coordinate(2,1:seq_num)))...
         /std(seq_coordinate(2,1:seq_num))/imgy_shrink + 5;
     for draw=1:seq_num  
         plot(x_coordinate(draw), y_coordinate(draw));
     end;
     h = gca;
     set(h, 'ylim', [-4, 8], 'xlim', [-2, 2]);
     
     %%% find out the corresponding points and draw lines between them in one axes
    
     width = 50; %window size
    % 
     pos_threshold = mean(TSProcess) + (max(TSProcess) - mean(TSProcess))/3;
     neg_threshold = mean(TSProcess) - (mean(TSProcess) - min(TSProcess))/3;
    % 
     TS = [0;0] ; % format the TimeSeries coordinate array
     LF = [0;0] ; % Leaf coordinate array
     count = 0;
     index = 2;
     while index < seq_num - width -1
         for run_index = 1:2
             if run_index == 1
                 [c, i] = max(TSProcess(index:index+width));
             else 
                 [c, i] = min(TSProcess(index:index+width));
             end;
             i = i+index-1;
             if (c > pos_threshold && TSProcess(i) > TSProcess(i-1)...
                 && TSProcess(i) > TSProcess(i+1))|| (c < neg_threshold &&...
                 TSProcess(i) < TSProcess(i-1)...
                 && TSProcess(i) < TSProcess(i+1))
                 count = count+1;
                 lf_x = seq_coordinate(1,i);
                 lf_y = seq_coordinate(2,i);
                 lf_x1 = (lf_x - mean(seq_coordinate(1,1:seq_num)))...
                         /std(seq_coordinate(1,1:seq_num))/imgx_shrink ;
                 lf_y1 = (lf_y - mean(seq_coordinate(2,1:seq_num)))...
                         /std(seq_coordinate(2,1:seq_num))/imgy_shrink + 5;
                 LF(:,count) = [lf_x1;lf_y1];
            
                 i1 = (i - mean(xaxes1))/std(xaxes1);
                 c1 = (c - mean(TSProcess))/std(TSProcess);
                 TS(:,count) = [i1;c1];
            
                 hold on,
                 line('XData', [lf_x1 i1], 'YData', [lf_y1 c]);
             end;
         end;
         index = index + width;
     end;
end;


% =========================================
function N = resampleAndNorm(T)

[row col] = size(T);
shortest = inf;
longest = 0;
for i = 1 : row
    if T(i,col) < shortest
        shortest = T(i,col);
    end
    if T(i,col) > longest
        longest = T(i,col);
    end
end
    
% resample to average length
length = round((shortest + longest) / 2);
N = zeros(row,length);
for i = 1 : row
    oldlen = T(i, end);
    seq = T(i,1:oldlen);
    N(i,:) = resample(seq, length, oldlen);
    N(i,:) = (N(i,:) - mean(N(i,:)))/std(N(i,:));   % normalize the data
end




